<?php
/**
 * Created by PhpStorm.
 * User: gb
 * Date: 1/19/2019
 * Time: 1:35 AM
 */

class WedoioInvVariantDetail extends WedoioEntity
{
    protected $invariant_src;
    protected $parent;

    public function __construct($params = false)
    {
        parent::__construct("InvVariantDetail",$params);
    }

    function get_original_invItem(){
        $api = new WedoioApi();
        $item = $this->_Item;
        $invitems_found = $api->fetch("InvItem",['_Item' => $item]);
        $invitem_found = reset($invitems_found);
        $parent = new WedoioInvitem();
        $parent->import($invitem_found);
        $this->parent = $parent;
        return $parent;
    }

    /**
     * Get the related Product from the system
     * @return WedoioProduct
     */
    function get_related_product(){
        $product = new WedoioProductVariation();
        $product->load_by_rowid($this->RowId);
        return $product;
    }

    public function sync(){
        $product = $this->get_related_product();
        if(!$this->parent) $this->get_original_invItem();
        $parent = $this->parent;
        $parent_product = $parent->get_related_product();

        if(!$parent_product->pid) {
            // No Parent product created yet. Weird ...
            return false;
        }

        $variations_attributes = [
            $this->_Variant1,
            $this->_Variant2,
            $this->_Variant3,
            $this->_Variant4,
            $this->_Variant5,
        ];

        // We check if there is a variation attribute
        $delete = true;
        foreach($variations_attributes as $attribute){
            if($attribute) {
                $delete = false;
                break;
            }
        }

        if($delete){
            // If there is not attributes, we just delete the product ... or skip it
            $product->delete();
            return false;
        }

        // We need to sync the variations attributes
        $attributes = [];
        $product_attributes = [];
        foreach($variations_attributes as $variant_id => $attribute){
            $invvariant = new WedoioInvVariant($variant_id + 1);
            $invvariant->load_by_variant($attribute);
            $invvariant->sync();
            $term = $invvariant->get_related_term();
            if($attribute) {
                $product_attributes[$term->taxonomy] = $term->term->slug;

                $product_attribute = new WC_Product_Attribute();
                $product_attribute->set_id(wc_attribute_taxonomy_id_by_name($term->taxonomy));
                $product_attribute->set_name( $term->taxonomy );
                $product_attribute->set_visible(false);
                $product_attribute->set_options([$term->term_id]);
                $product_attribute->set_variation(true);

                $attributes[$term->taxonomy] = $product_attribute;
            }
        }

        // We need to set the parent product attributes too. So we need to merge the new terms and any previous one that was there
        $current_parent_attributes = $parent_product->product->get_attributes();
        foreach($attributes as $name => $values){
            if(!isset($current_parent_attributes[$name])) $current_parent_attributes[$name] = $values;
        }

        foreach($current_parent_attributes as $name => $values){
            $new_options = isset($attributes[$name]) ? $attributes[$name]->get_options() : [];
            $current_options = $values->get_options();
            $options = array_merge($current_options,$new_options);
            $options = array_unique($options);
            rsort($options);
            $values->set_options($options);

            wp_set_object_terms( $parent_product->pid, $options, $name );
        }

        $parent_product->product->set_attributes($current_parent_attributes);
        $parent_product->product->save();

        // Now normally the product attributes are registered in the system. We need to add them to the product
        $sku = str_replace(";","_",$this->KeyName);

        $mapping = $parent->mapping();

        $p = $product->product;
        $p->set_parent_id($parent_product->pid);
        $p->set_attributes($product_attributes);
        $p->set_sku($sku);

        if(isset($mapping['_price'])){
            $p->set_price($mapping['_price']);
        }

        if(isset($mapping['_regular_price'])){
            $p->set_regular_price($mapping['_regular_price']);
        }

        if(isset($mapping['_sale_price'])){
            $p->set_sale_price($mapping['_sale_price']);
        }

        $p->save();
        $product->product = $p;

        if(!is_wp_error($p)){
            $pid = $p->get_id();
            update_post_meta($pid,"_uniconta-rowid",$this->RowId);
            return $pid;
        }

        return false;
    }
}